/*
 * vi: formatoptions+=tc textwidth=80 tabstop=8 shiftwidth=8 noexpandtab:
 *
 * $Id: consnoop.cpp,v 1.10 2008/04/11 05:05:26 jaymzh Exp $
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * (C) Copyright Kevin Timmerman 2007
 * (C) Copyright Phil Dibowitz 2007
 */

#include <string>
#include <fstream>
using namespace std;

#include "../libconcord/protocol.h"

#ifdef WIN32
typedef unsigned char uint8_t;
#endif

int HexToInt(char c)
{
	if(c>='0' && c<='9') return c-'0';
	if(c>='a' && c<='f') return c-'a'+10;
	if(c>='A' && c<='F') return c-'A'+10;
	return -1;
}

const char* GetMisc(uint8_t x)
{
	switch(x) {
		case COMMAND_MISC_EEPROM:
			return "EEPROM";
		case COMMAND_MISC_STATE:
			return "State";
		case COMMAND_MISC_INVALIDATE_FLASH:
			return "Invalidate Flash";
		case COMMAND_MISC_QUEUE_ACTION:
			return "Queue Action";
		case COMMAND_MISC_PROGRAM:
			return "Program";
		case COMMAND_MISC_INTERRUPT:
			return "Interrupt";
		case COMMAND_MISC_RAM:
			return "RAM";
		case COMMAND_MISC_REGISTER:
			return "Register";
		case COMMAND_MISC_CLOCK_RECALCULATE:
			return "Clock Recalculate";
		case COMMAND_MISC_QUEUE_EVENT:
			return "Queue Event";
		case COMMAND_MISC_RESTART_CONFIG:
			return "Restart Config";
	}
	return "Unknown";
}

void Decode(const uint8_t * const data)
{
	// Note: The uninteresting stuff is commented out
	const uint8_t length=data[0] & LENGTH_MASK;
	switch(data[0] & COMMAND_MASK) {
		case COMMAND_GET_VERSION & COMMAND_MASK:
			printf("Get   Version\n");
			break;
		case RESPONSE_VERSION_DATA & COMMAND_MASK:
			printf("Get   Version Response %02X %02X %02X %02X %02X %02X %02X\n",
				data[1],data[2],data[3],data[4],data[5],data[6],data[7]);
			break;
		case COMMAND_WRITE_FLASH & COMMAND_MASK:
			printf("Write Flash %06X %i\n",data[1]<<16|data[2]<<8|data[3],data[4]<<8|data[5]);
			break;
		case COMMAND_WRITE_FLASH_DATA & COMMAND_MASK:
			//printf("Write Flash Data\n");
			break;
		case COMMAND_READ_FLASH & COMMAND_MASK:
			printf("Read  Flash %06X %i\n",data[1]<<16|data[2]<<8|data[3],data[4]<<8|data[5]);
			break;
		case RESPONSE_READ_FLASH_DATA & COMMAND_MASK:
			//printf("Read  Flash Data\n");
			break;
		case COMMAND_START_IRCAP & COMMAND_MASK:
			printf("Start IR capture\n");
			break;
		case COMMAND_STOP_IRCAP & COMMAND_MASK:
			printf("Stop  IR capture\n");
			break;
		case RESPONSE_IRCAP_DATA & COMMAND_MASK:
			//printf("IR Capture Data\n");
			break;
		case COMMAND_WRITE_MISC & COMMAND_MASK:
			if(length==1)
				printf("Write %s\n",GetMisc(data[1]));
			else if(length==3)
				printf("Write %s %02X %02X\n",
					GetMisc(data[1]),data[2],data[3]);
			else
				printf("Write %s %02X %02X %02X %02X\n",
					GetMisc(data[1]),data[2],data[3],data[4],data[5]);
			break;
		case COMMAND_READ_MISC & COMMAND_MASK:
			if(length==2)
				printf("Read  %s %02X\n",GetMisc(data[1]),data[2]);
			else
				printf("Read  %s %02X %02X\n",GetMisc(data[1]),data[2],data[3]);
			break;
		case RESPONSE_READ_MISC_DATA & COMMAND_MASK:
			/*
			if(length==2)
				printf("Read  %s Data %02X\n",GetMisc(data[1]),data[2]);
			else
				printf("Read  %s Data %02X %02X\n",GetMisc(data[1]),data[2],data[3]);
			*/
			break;
		case COMMAND_ERASE_FLASH & COMMAND_MASK:
			printf("Erase Flash %06X\n",data[1]<<16|data[2]<<8|data[3]);
			break;
		case COMMAND_RESET & COMMAND_MASK:
			printf("Reset ");
			switch(data[1]) {
				case COMMAND_RESET_USB:
					printf("USB (01)\n");
					break;
				case COMMAND_RESET_DEVICE:
					printf("Device (02)\n");
					break;
				case COMMAND_RESET_DEVICE_DISCONNECT:
					printf("Device Disconnect (03)\n");
					break;
				case COMMAND_RESET_TEST_FINISH:
					printf("Test Finish (04)\n");
					break;
				default:
					printf("Unknown (%02X)\n",data[1]);
					break;
			}
			break;
		case COMMAND_DONE & COMMAND_MASK:
			//printf("Done %02X\n",data[1]);
			break;
	}
}

int main(int argc, char *argv[])
{
	char *file_name=NULL;
	for(int a=1; a<argc; ++a) {
		if(argv[a][0]!='-' && file_name==NULL) file_name=argv[a];
	}
	if (file_name==NULL)
		file_name = (char *)"Log.xml";

	ifstream infile;
	infile.open(file_name);

	string s;
	string payloadbytes("<payloadbytes>");

	while(!infile.eof()) {
		getline(infile,s);
		if(!s.compare(0,payloadbytes.size(),payloadbytes)) {
			uint8_t data[260];
			unsigned int n=0;
			for(unsigned int i=payloadbytes.size(); i<s.size(); ++i) {
				const char c=s[i];
				if(c=='<') break;
				const int h=HexToInt(c);
				if(h<0) continue;
				if(n&1)
					data[n>>1]|=h;
				else
					data[n>>1]=h<<4;
				++n;
				if(n>=sizeof(data)*2) break;
			}
			n>>=1;
			if(!n) continue;
			//for(i=0; i<n; ++i) printf("%02X",data[i]); printf("\n"); // Show all payload bytes for debugging
			if(data[0]==0x12) continue; // hack - Ignore USB descriptor
			Decode(data);
		}
	}
	infile.close();
	return 0;
}
